#include "DoubleButton.hpp"

#include <QHBoxLayout>
#include <QPushButton>
#include <QSizePolicy>

DoubleButton::DoubleButton(QString label1, QString label2, QWidget *parent)
	: QWidget(parent)
{
	QHBoxLayout *layout = new QHBoxLayout(this);
	left = new QPushButton(label1, this);
	right = new QPushButton(label2, this);

	left->setStyleSheet("QPushButton { "
		"border-top-right-radius: 0;"
		"border-bottom-right-radius: 0;"
		"text-align: center;"
	"}");
	right->setStyleSheet("QPushButton { "
		"border-top-left-radius: 0;"
		"border-bottom-left-radius: 0;"
		"border-left-width: 0;"
		"text-align: center;"
	"}");

	layout->addWidget(left);
	layout->addWidget(right);

	connect(left, SIGNAL(clicked()), this, SIGNAL(leftClicked()));
	connect(right, SIGNAL(clicked()), this, SIGNAL(rightClicked()));

	layout->setSizeConstraint(QLayout::SetFixedSize);
	layout->setSpacing(0);
}

void DoubleButton::resizeEvent(QResizeEvent *)
{
	int maximum_width = 0;

	maximum_width = left->minimumSizeHint().width();
	if(right->minimumSizeHint().width() > maximum_width)
		maximum_width = right->minimumSizeHint().width();

	left->setFixedWidth(maximum_width);
	right->setFixedWidth(maximum_width);
}
