#include "DropArea.hpp"

DropArea::DropArea(QWidget *parent) : QLabel(parent)
{
	setAlignment(Qt::AlignCenter);
	setAcceptDrops(true);
	setAutoFillBackground(true);
	setResult(Analysis_Done);
}

void DropArea::setResult(AnalysisResult result, QString message)
{
	QString backgrounds[3] = { "#212223", "#4c2223", "#892223" };
	setStyleSheet("DropArea { background: " + backgrounds[result] + "; }");

	if(result == Analysis_Done) setText("Drag and drop an image here.");
	else setText(message);
}

void DropArea::dragEnterEvent(QDragEnterEvent *event)
{
	setStyleSheet("DropArea { background: #414243; }");
	event->acceptProposedAction();
}

void DropArea::dropEvent(QDropEvent *event)
{
	const QMimeData *mimeData = event->mimeData();
	QImage image;

	setStyleSheet("DropArea { background: #212223; }");
	event->acceptProposedAction();

	// Getting the image from the mime data.
	if(mimeData->hasImage())
	{
		image = qvariant_cast<QPixmap>(mimeData->imageData())
			.toImage();
	}
	// Or from an URL when dragging a file.
	else
	{
		QList<QUrl> urls = mimeData->urls();
		
		if(urls.count())
		{
			QString url = mimeData->urls()[0].toString();
#ifdef _WIN32
			if(url.left(8) == "file:///")
				url = url.right(url.length() - 8 );
#else
			if(url.left(7) == "file://")
				url = url.right(url.length() - 7);
#endif
			image.load(url);
		}
	}

	if(image.isNull())
	{
		setResult(Analysis_Error, "No image could be loaded.");
		return;
	}

	emit dropped(image);
}

void DropArea::dragLeaveEvent(QDragLeaveEvent *)
{
	setStyleSheet("DropArea { background: #212223; }");
}
