#include "ImageEntry.hpp"
#include "DoubleButton.hpp"
#include <stdlib.h>

ImageEntry::ImageEntry(QImage img, void *ml, void* var, QString info,
	QString subinfo, QWidget *parent)
	: QWidget(parent), _ml_code(ml), _variant(var)
{
	_pixmap = QPixmap::fromImage(img);

	QLabel *image = new QLabel(this);
	image->setPixmap(_pixmap);

	QLabel *label = new QLabel(this);
	label->setTextFormat(Qt::RichText);
	label->setText(info + "<br /><font color='#818283'>" + subinfo
		+ "</font>");
	label->setAlignment(Qt::AlignHCenter | Qt::AlignVCenter);
	label->setStyleSheet("QLabel { font-size: 10px; }");

	QWidget *button_copy;

	if(_variant)
	{
		button_copy = new DoubleButton("ML code", "Variant", this);
		connect(button_copy, SIGNAL(leftClicked()), this,
			SLOT(generate_ml()));
		connect(button_copy, SIGNAL(rightClicked()), this,
			SLOT(generate_variant()));
	}
	else
	{
		button_copy = new QPushButton("Map data", this);
		connect(button_copy, SIGNAL(clicked()), this,
			SLOT(generate_ml()));
	}

	QHBoxLayout *layout = new QHBoxLayout(this);
	layout->addWidget(image);
	layout->addWidget(label);
	layout->addStretch();
	layout->addWidget(button_copy);
}

QPixmap ImageEntry::pixmap(void)
{
	return _pixmap;
}

void *ImageEntry::ml_code(void)
{
	return _ml_code;
}

void *ImageEntry::variant(void)
{
	return _variant;
}

void ImageEntry::generate_ml(void)
{
	if(!_ml_code) return;
	emit generate(_ml_code);
}

void ImageEntry::generate_variant(void)
{
	if(!_variant) return;
	emit generate(_variant);
}
