#include "MainWindow.hpp"
#include "encoding.hpp"
#include "generating.hpp"
#include <stdlib.h>

#include <QDialog>
#include <QDialogButtonBox>
#include <QFormLayout>
#include <QHBoxLayout>
#include <QLabel>
#include <QPushButton>
#include <QRadioButton>
#include <QToolButton>
#include <QVBoxLayout>

MainWindow::MainWindow(void)
{
	setupAdvancedInterface();

	setWindowTitle("fxSpriter");

	setMinimumSize(600, 400);
	resize(800, 400);
}



/***
	Basic public slots.
***/



void MainWindow::help(void)
{
	QDialog dialog;
	QDialogButtonBox box(&dialog);
	QVBoxLayout layout(&dialog);
	QPushButton button("Ok", &dialog);

	QLabel label(
		"fxSpriter is a tool that converts black and white images"
		" to raw data for fx-9860g programs.\n"
		"Supported formats : BMP GIF JPG JPEG PNG PBM PGM PPM TIFF XBM"
		" XPM.",
		&dialog
	);
	label.setWordWrap(true);

	box.addButton(&button, QDialogButtonBox::AcceptRole);
	QObject::connect(&button, SIGNAL(clicked()), &dialog, SLOT(close()));

	layout.addWidget(&label);
	layout.addWidget(&box);
	dialog.setFixedWidth(400);
	dialog.exec();
}

void MainWindow::modeImage(void)
{
	advanced->setCurrentIndex(0);
}

void MainWindow::modeSet(void)
{
	advanced->setCurrentIndex(1);
}

void MainWindow::modeMap(void)
{
	advanced->setCurrentIndex(2);
}



/***
	Image encoding and access.
***/



void  MainWindow::encodeImage(QImage image)
{
	struct Image *ml_code = new struct Image();
	struct Image *variant = new struct Image();
	QString info("%1 x %2");
	QString subinfo("%1 bytes");

	switch(encode_image(image, ml_code, variant))
	{
	case -1:
		image_drop->setResult(DropArea::Analysis_Error, 
			"Alloc failure.");
		return;
	case 0:
		image_drop->setResult(DropArea::Analysis_Done);
		break;
	case 1:
		image_drop->setResult(DropArea::Analysis_Error,
			"Empty image.");
		return;
	}

	info = info.arg(image.width()).arg(image.height());
	subinfo = subinfo.arg(ml_code->size);

	ImageEntry *entry = new ImageEntry(image, ml_code, variant, info,
		subinfo, this);
	connect(entry, SIGNAL(generate(void *)), this,
		SLOT(copyImage(void *)));

	QListWidgetItem *item = new QListWidgetItem();
	image_list->insertItem(0, item);
	image_list->setItemWidget(item, entry);
	item->setSizeHint(entry->sizeHint());
}

void MainWindow::copyImage(void *v_image)
{
	struct Image *image = (struct Image *)v_image;

	QString code = QString("static const unsigned char image[%1] = ");
	code = code.arg(image->size);
	code += generate(image->data, image->size, image_spaces->checkState(),
		image_wrap80->checkState());
	code += ";\n";

	QGuiApplication::clipboard()->setText(code);
}

void MainWindow::encodeSet(QImage image)
{
	struct Set *ml_code = new struct Set();
	struct Set *variant = new struct Set();
	QString info("%1 elements<br />%2 x %3");
	QString subinfo("%1 bytes");
	bool valid;
	int width, height;

	width = set_width->text().toInt(&valid);
	if(!valid) return;
	height = set_height->text().toInt(&valid);
	if(!valid) return;

	switch(encode_set(image, width, height, (enum BorderKind)
		set_borders->checkedId(), ml_code, variant))
	{
	case -1:
		set_drop->setResult(DropArea::Analysis_Error,
			"Alloc failure.");
		return;
	case 0:
		set_drop->setResult(DropArea::Analysis_Done);
		break;
	case 1:
		set_drop->setResult(DropArea::Analysis_Error, "Empty set.");
		return;
	}


	info = info.arg(ml_code->elements).arg(width).arg(height);
	subinfo = subinfo.arg(ml_code->elements * ml_code->element_size);

	ImageEntry *entry = new ImageEntry(image, ml_code, variant, info,
		subinfo, this);
	connect(entry, SIGNAL(generate(void *)), this, SLOT(copySet(void *)));

	QListWidgetItem *item = new QListWidgetItem();
	set_list->insertItem(0, item);
	set_list->setItemWidget(item, entry);
	item->setSizeHint(entry->sizeHint());
}

void MainWindow::copySet(void *v_set)
{
	struct Set *set = (struct Set *)v_set;
	int n;

	QString code = QString("static const unsigned char set[%1][%2] = {\n");
	code = code.arg(set->elements).arg(set->element_size);

	for(n = 0; n < set->elements; n++)
	{
		code += '\t';
		code += generate(set->data[n], set->element_size, 1, 0);
		code += ",\n";
	}
	code += "};\n";

	QGuiApplication::clipboard()->setText(code);
}

void MainWindow::encodeMap(QImage image)
{
	QListWidgetItem *item = set_list->currentItem();
	ImageEntry *entry = (ImageEntry *)set_list->itemWidget(item);
	if(!entry) return;
	QString info("Map of %1 x %2<br />%3 x %4");
	QString subinfo("%1 bytes");

	struct Map *map = new struct Map();
	struct Set *set = (struct Set *)entry->ml_code();

	switch(encode_map(image, set, map))
	{
	case -1:
		map_drop->setResult(DropArea::Analysis_Error,
			"Alloc failure.");
		return;
	case 0:
		map_drop->setResult(DropArea::Analysis_Done);
		break;
	case 1:
		map_drop->setResult(DropArea::Analysis_Error, "Empty map.");
		return;
	case 2:
		map_drop->setResult(DropArea::Analysis_Warning,
			"Some tiles are missing.");
		break;
	}

	info = info.arg(map->width).arg(map->height);
	info = info.arg(map->width * set->element_width);
	info = info.arg(map->height * set->element_height);
	subinfo = subinfo.arg(map->elements);

	entry = new ImageEntry(image, map, NULL, info, subinfo, this);
	connect(entry, SIGNAL(generate(void *)), this, SLOT(copyMap(void *)));

	item = new QListWidgetItem();
	map_list->insertItem(0, item);
	map_list->setItemWidget(item, entry);
	item->setSizeHint(entry->sizeHint());
}

void MainWindow::copyMap(void *v_map)
{
	struct Map *map = (struct Map *)v_map;

	QString code = QString("static const unsigned char map[%1] = ");
	code = code.arg(map->elements);
	code += generate(map->data, map->width * map->height, 1, 0);
	code += ";\n";

	QGuiApplication::clipboard()->setText(code);
}



/***
	Tileset selection for map encoding.
***/


void MainWindow::updateSelectedTileset(QListWidgetItem *item)
{
	ImageEntry *image = (ImageEntry *)set_list->itemWidget(item);
	if(!image) return;

	map_tileset->setPixmap(image->pixmap());
}



/***
	Interface setup.
***/



void MainWindow::setupAdvancedInterface(void)
{
	// Setting up the various parts of the advanced interface.

	QWidget *mode = new QWidget(this);
	QVBoxLayout *mode_layout = new QVBoxLayout(mode);

	advanced = new QStackedLayout();

	QHBoxLayout *layout = new QHBoxLayout(this);
	layout->addWidget(mode);
	layout->addLayout(advanced);

	// Setting up the encoding mode toolbar.

	QToolButton *mode_image = new QToolButton(this);
	mode_image->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
	mode_image->setIcon(QIcon("images/image.png"));
	mode_image->setIconSize(QSize(33, 29));
	mode_image->setText("Image");
	mode_image->setFocusPolicy(Qt::NoFocus);
	mode_layout->addWidget(mode_image);
	connect(mode_image, SIGNAL(clicked()), this, SLOT(modeImage()));

	QToolButton *mode_set = new QToolButton(this);
	mode_set->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
	mode_set->setIcon(QIcon("images/set.png"));
	mode_set->setIconSize(QSize(33, 29));
	mode_set->setText("Set");
	mode_set->setFocusPolicy(Qt::NoFocus);
	mode_layout->addWidget(mode_set);
	connect(mode_set, SIGNAL(clicked()), this, SLOT(modeSet()));

	QToolButton *mode_map = new QToolButton(this);
	mode_map->setToolButtonStyle(Qt::ToolButtonTextUnderIcon);
	mode_map->setIcon(QIcon("images/map.png"));
	mode_map->setIconSize(QSize(33, 29));
	mode_map->setText("Map");
	mode_map->setFocusPolicy(Qt::NoFocus);
	mode_layout->addWidget(mode_map);
	connect(mode_map, SIGNAL(clicked()), this, SLOT(modeMap()));

	mode_layout->addStretch();

	QPushButton *mode_help = new QPushButton("Help", this);
	mode_layout->addWidget(mode_help);
	connect(mode_help, SIGNAL(clicked()), this, SLOT(help()));

	// Setting up the various interfaces.
	setupImageInterface();
	setupSetInterface();
	setupMapInterface();

	modeImage();
}

void MainWindow::setupImageInterface(void)
{
	QWidget *image = new QWidget(this);
	QHBoxLayout *layout = new QHBoxLayout(image);
	advanced->addWidget(image);

	QVBoxLayout *subLayout = new QVBoxLayout();
	layout->addLayout(subLayout, 1);

	image_drop = new DropArea(this);
	connect(image_drop, SIGNAL(dropped(QImage)), this,
		SLOT(encodeImage(QImage)));

	subLayout->addWidget(image_drop, 1);
	subLayout->addSpacing(16);

	image_spaces = new QCheckBox("Add spaces between values", this);
	image_spaces->setChecked(true);
	image_wrap80 = new QCheckBox("Wrap at 80 characters", this);
	subLayout->addWidget(image_spaces);
	subLayout->addWidget(image_wrap80);

	image_list = new QListWidget(this);
	image_list->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
	layout->addWidget(image_list, 2);
}

void MainWindow::setupSetInterface(void)
{
	QWidget *set = new QWidget(this);
	QHBoxLayout *layout = new QHBoxLayout(set);
	advanced->addWidget(set);

	QVBoxLayout *subLayout = new QVBoxLayout();
	layout->addLayout(subLayout, 1);

	DropArea *drop = new DropArea(this);
	connect(drop, SIGNAL(dropped(QImage)), this, SLOT(encodeSet(QImage)));

	subLayout->addWidget(drop, 1);
	subLayout->addSpacing(16);

	QFormLayout *sizeLayout = new QFormLayout();
	sizeLayout->setFieldGrowthPolicy(QFormLayout::FieldsStayAtSizeHint);
	subLayout->addLayout(sizeLayout);
	subLayout->addSpacing(16);
	set_width = new QLineEdit(this);
	set_height = new QLineEdit(this);
	sizeLayout->addRow("Element width", set_width);
	sizeLayout->addRow("Element height", set_height);
	sizeLayout->setContentsMargins(0, 0, 16, 0);

	set_borders = new QButtonGroup(this);
	QRadioButton *borders_0 = new QRadioButton("  No borders", this);
	QRadioButton *borders_1 = new QRadioButton("  Separation", this);
	QRadioButton *borders_2 = new QRadioButton("  Full borders", this);
	borders_0->setIcon(QIcon("images/tiles0.png"));
	borders_1->setIcon(QIcon("images/tiles1.png"));
	borders_2->setIcon(QIcon("images/tiles2.png"));
	borders_0->setIconSize(QSize(33, 21));
	borders_1->setIconSize(QSize(33, 21));
	borders_2->setIconSize(QSize(33, 21));
	set_borders->addButton(borders_0, 0);
	set_borders->addButton(borders_1, 1);
	set_borders->addButton(borders_2, 2);
	subLayout->addWidget(borders_0);
	subLayout->addWidget(borders_1);
	subLayout->addWidget(borders_2);
	borders_2->setChecked(true);

	set_list = new QListWidget(this);
	set_list->setSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum);
	connect(set_list, SIGNAL(currentItemChanged(QListWidgetItem *,
		QListWidgetItem *)), this, SLOT(updateSelectedTileset(
		QListWidgetItem *)));
	layout->addWidget(set_list, 2);
}

void MainWindow::setupMapInterface(void)
{
	QWidget *widget = new QWidget(this);
	QHBoxLayout *layout = new QHBoxLayout(widget);
	advanced->addWidget(widget);

	QVBoxLayout *subLayout = new QVBoxLayout();
	layout->addLayout(subLayout, 1);

	map_drop = new DropArea(this);
	connect(map_drop, SIGNAL(dropped(QImage)), this,
		SLOT(encodeMap(QImage)));
	subLayout->addWidget(map_drop, 1);
	subLayout->addSpacing(16);

	QLabel *label = new QLabel("Selected tileset:", this);
	subLayout->addWidget(label);

	map_tileset = new QLabel("None", this);
	subLayout->addWidget(map_tileset);

	map_list = new QListWidget(this);
	layout->addWidget(map_list, 2);
}
