#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QButtonGroup>
#include <QCheckBox>
#include <QLineEdit>
#include <QListWidget>
#include <QListWidgetItem>
#include <QMimeData>
#include <QPixmap>
#include <QPlainTextEdit>
#include <QStackedLayout>
#include <QTableWidget>
#include <QWidget>

#include "DropArea.hpp"
#include "ImageEntry.hpp"

class MainWindow : public QWidget
{
	Q_OBJECT

public:
	MainWindow(void);

public slots:
	void help(void);

	void modeImage(void);
	void modeSet(void);
	void modeMap(void);

	void encodeImage(QImage image);
	void copyImage(void *image);
	void encodeSet(QImage image);
	void copySet(void *set);
	void encodeMap(QImage image);
	void copyMap(void *map);

private slots:
	void updateSelectedTileset(QListWidgetItem *item);

private:
	void setupAdvancedInterface(void);
	void setupImageInterface(void);
	void setupSetInterface(void);
	void setupMapInterface(void);

	QStackedLayout *advanced;

	QListWidget *image_list;
	DropArea *image_drop;
	QCheckBox *image_spaces;
	QCheckBox *image_wrap80;

	QButtonGroup *set_borders;
	DropArea *set_drop;
	QLineEdit *set_width;
	QLineEdit *set_height;
	QListWidget *set_list;

	QLabel *map_tileset;
	DropArea *map_drop;
	QListWidget *map_list;
};

#endif
